@@ -28,6 +28,11 @@ show_segment(struct segment *self,
              char *str
             );
 
+void
+shrink_segment(struct segment *self,
+               PyArrayObject *pixmap,
+               int jdim);
+
 void
 sort_segment(struct segment *self,
              int jdim
